###############################################################
# Copyright (c) 2007 Xilinx, Inc. All Rights Reserved.
#
# You may copy and modify these files for your own internal use solely with
# Xilinx programmable logic devices and  Xilinx EDK system or create IP
# modules solely for Xilinx programmable logic devices and Xilinx EDK system.
# No rights are granted to distribute any files unless they are distributed in
# Xilinx programmable logic devices. 
#
###############################################################


#---------------------------------------------
# ISF_drc - check system configuration and make sure 
# all components to run ISF are available. 
#---------------------------------------------
proc isf_drc {libhandle} {
	puts "Runnning DRC for ISF library... \n"

	# find the list of xps or opb spi cores
	set sw_processor [xget_libgen_proc_handle]
	set processor [xget_handle $sw_processor "IPINST"]

	set spi_periphs_list [get_spi_periphs $processor]

	if { [llength $spi_periphs_list] == 0 } {
		set cpuname [xget_hw_name $processor]
		error "No SPI core is addressable from processor $cpuname. \
			ISF requires SPI Core \n"
		return
	} 
	
	#----
	# check SPI for correctness conditions
	#----
	
	isf_hw_drc $libhandle $spi_periphs_list
}

proc get_spi_periphs {processor} {
	set periphs_list [xget_hw_proc_slave_periphs $processor]
	set spi_periphs_list {}

	foreach periph $periphs_list {
		set periphname [xget_hw_value $periph]
		if {$periphname == "xps_spi" 
			|| $periphname == "opb_spi"} {
			lappend spi_periphs_list $periph	
		}
	}

	return $spi_periphs_list
}

#--------
# Check the following h/w requirements for ISF:
#	1. interrupts should be connected:
#		- for SPI, SPI interrupt should be connected
#--------
proc isf_hw_drc {libhandle spi_list} {
	foreach ip $spi_list {
		set iptype [xget_hw_value $ip]
		if {$iptype == "xps_spi"} {
			isf_spi_hw_drc $libhandle $ip
		} elseif {$iptype == "opb_spi"} {
			isf_spi_hw_drc $libhandle $ip
		}
	}
}


# SPI hw requirements - interrupt needs to be connected
proc isf_spi_hw_drc {libhandle spi} {
	set spiname [xget_hw_name $spi]
	set intr_port [xget_value $spi "PORT" "IP2INTC_Irpt"]
	if {[string compare -nocase $intr_port ""] == 0} {
		error "SPI core $spi does not have its interrupt connected to interrupt controller. \
			ISF operates only in interrupt mode, so please connect the interrupt port to \
			the interrupt controller.\n" "" "MDT_ERROR"
	}
}



#-------
# post_generate: called after generate called on all libraries
#-------
proc post_generate {libhandle} {
}

#-------
# execs_generate: called after BSP's, libraries and drivers have been compiled
#	This procedure builds the libisf.a library
#-------
proc execs_generate {libhandle} {
}
