/* $Id: xilisf_erase.c 1.1 2006/02/17 21:21:36 mta Exp $ */
/******************************************************************************
*
*       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
*       SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
*       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
*       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
*       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*       FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*       FOR A PARTICULAR PURPOSE.
*
*       (c) Copyright 2007 Xilinx Inc.
*       All rights reserved.
*
******************************************************************************/
/*****************************************************************************/
/**
*
* @file xilisf_erase.c
*
* This file contains the library functions to erase the In System Flash(ISF)
* device. Refer xilisf.h for detailed description.
*
* <pre>
*
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -----------------------------------------------
* 1.00a mta  07/11/07 First release.
*
* </pre>
*
******************************************************************************/

/***************************** Include Files *********************************/

#include "includes/xilisf.h"

/************************** Constant Definitions *****************************/

/**************************** Type Definitions *******************************/

/***************** Macros (Inline Functions) Definitions *********************/

/************************** Function Prototypes ******************************/

/************************** Variable Definitions *****************************/

/************************** Function Definitions ******************************/
extern int IsfTransfer(u8 *WritePtr, u8* ReadPtr,u32 ByteCount);
/*****************************************************************************/
/**
*
* This function erases the contents of the specified Page in the ISF.
*
* @param	Addr is the address of the page to be erased.
*
* @return	XST_SUCCESS if successful else XST_FAILURE.
*
* @note		This works for frequencies up to 66MHz. The erased bytes will
*		read as 0xFF.
*		Read the Spartan-3AN In-System Flash User Guide for more
*		information.
*
******************************************************************************/
int XIsf_PageErase(XIsf_Address *Addr)
{
	int Status;

	if(IsfDev.IsReady != TRUE) {
		return XST_FAILURE;
	}

	if(Addr == NULL) {
		return XST_FAILURE;
	}

	if(Addr->SectorAddr > (IsfDev.NumOfSectors - 1)) {
		return XST_FAILURE;
	}

	if(Addr->BlockAddr > (IsfDev.BlocksPerSector - 1)) {
		return XST_FAILURE;
	}

	if(Addr->PageAddr > (IsfDev.PagesPerBlock - 1)) {
		return XST_FAILURE;
	}

	/*
	 * Prepare the Write Buffer.
	 */
	IsfDev.IsfWriteBuffer[BYTE0] = XISF_COMMAND_PAGE_ERASE;
	IsfDev.IsfWriteBuffer[BYTE1] = (Addr->SectorAddr <<
					XISF_SECTORADDR_BYTE_SHIFT_MASK) |
					(Addr->BlockAddr >>
					XISF_BLOCKADDR_BYTE1_SHIFT_MASK);

	IsfDev.IsfWriteBuffer[BYTE2] = (Addr->BlockAddr <<
					XISF_BLOCKADDR_BYTE2_SHIFT_MASK) |
			  		(Addr->PageAddr <<
			  		XISF_PAGEADDR_BYTE2_SHIFT_MASK);

	IsfDev.IsfWriteBuffer[BYTE3] = XISF_DUMMYBYTE; /* Dummy byte. */

	/*
	 * Initiate the Transfer.
	 */
	Status = IsfTransfer(IsfDev.IsfWriteBuffer, NULL,
						XISF_ISF_PAGE_ERASE_BYTES);
	if(Status != XST_SUCCESS) {
		return XST_FAILURE;
	}
	return XST_SUCCESS;
}

/*****************************************************************************/
/**
*
* This function erases the content of the specified Block in the ISF.
*
* @param	Addr is the address of the block to be erased.
*
* @return	XST_SUCCESS if successful else XST_FAILURE.
*
* @note		This works for frequencies up to 66MHz. The erased bytes will
*		read as 0xFF.
*		Read the Spartan-3AN In-System Flash User Guide for more
*		information.
*
******************************************************************************/
int XIsf_BlockErase(XIsf_Address *Addr)
{
	int Status;

	if(IsfDev.IsReady != TRUE) {
		return XST_FAILURE;
	}

	if(Addr == NULL) {
		return XST_FAILURE;
	}

	if(Addr->SectorAddr > (IsfDev.NumOfSectors - 1)) {
		return XST_FAILURE;
	}

	if(Addr->BlockAddr > (IsfDev.BlocksPerSector - 1)) {
		return XST_FAILURE;
	}

	if(Addr->PageAddr > (IsfDev.PagesPerBlock - 1)) {
		return XST_FAILURE;
	}

	/*
	 * Prepare the Write Buffer.
	 */
	IsfDev.IsfWriteBuffer[BYTE0] = XISF_COMMAND_BLOCK_ERASE;
	IsfDev.IsfWriteBuffer[BYTE1] = (Addr->SectorAddr <<
					XISF_SECTORADDR_BYTE_SHIFT_MASK) |
					(Addr->BlockAddr >>
					XISF_BLOCKADDR_BYTE1_SHIFT_MASK);

	IsfDev.IsfWriteBuffer[BYTE2] = (Addr->BlockAddr <<
					XISF_BLOCKADDR_BYTE2_SHIFT_MASK) |
			  		(Addr->PageAddr <<
			  		XISF_PAGEADDR_BYTE2_SHIFT_MASK);

	IsfDev.IsfWriteBuffer[BYTE3] = XISF_DUMMYBYTE; /* Dummy byte. */

	/*
	 * Initiate the Transfer.
	 */
	Status = IsfTransfer(IsfDev.IsfWriteBuffer, NULL,
						XISF_ISF_BLOCK_ERASE_BYTES);
	if(Status != XST_SUCCESS) {
		return XST_FAILURE;
	}

	return XST_SUCCESS;
}

/*****************************************************************************/
/**
*
* This function erases the content of the specified Sector in he ISF.
*
* @param	Addr is the address in the Buffer, where to write the data.
*
* @return	XST_SUCCESS if successful else XST_FAILURE.
*
* @note		This works for frequencies up to 66MHz. The erased bytes will
*		read as 0xFF.
*		Read the Spartan-3AN In-System Flash User Guide for more
*		information.
*
******************************************************************************/
int XIsf_SectorErase(XIsf_Address *Addr)
{
	int Status;

	if(IsfDev.IsReady != TRUE) {
		return XST_FAILURE;
	}

	if(Addr == NULL) {
		return XST_FAILURE;
	}

	if(Addr->SectorAddr > (IsfDev.NumOfSectors - 1)) {
		return XST_FAILURE;
	}

	if(Addr->BlockAddr > (IsfDev.BlocksPerSector - 1)) {
		return XST_FAILURE;
	}

	if(Addr->PageAddr > (IsfDev.PagesPerBlock - 1)) {
		return XST_FAILURE;
	}

	/*
	 * Prepare the Write Buffer.
	 */
	IsfDev.IsfWriteBuffer[BYTE0] = XISF_COMMAND_SECTOR_ERASE;
	IsfDev.IsfWriteBuffer[BYTE1] = (Addr->SectorAddr <<
					XISF_SECTORADDR_BYTE_SHIFT_MASK) |
					(Addr->BlockAddr >>
			 		XISF_BLOCKADDR_BYTE1_SHIFT_MASK);

	IsfDev.IsfWriteBuffer[BYTE2] = (Addr->BlockAddr <<
					XISF_BLOCKADDR_BYTE2_SHIFT_MASK) |
			  		(Addr->PageAddr <<
			  		XISF_PAGEADDR_BYTE2_SHIFT_MASK);

	IsfDev.IsfWriteBuffer[BYTE3] = XISF_DUMMYBYTE; /* Dummy byte. */

	/*
	 * Initiate the Transfer.
	 */
	Status = IsfTransfer(IsfDev.IsfWriteBuffer, NULL,
						XISF_ISF_SECTOR_ERASE_BYTES);
	if(Status != XST_SUCCESS) {
		return XST_FAILURE;
	}

	return XST_SUCCESS;
}
