/* $Id: xilisf_read.c 1.1 2006/02/17 21:21:36 mta Exp $ */
/******************************************************************************
*
*       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
*       SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
*       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
*       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
*       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*       FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*       FOR A PARTICULAR PURPOSE.
*
*       (c) Copyright 2007 Xilinx Inc.
*       All rights reserved.
*
******************************************************************************/
/*****************************************************************************/
/**
*
* @file xilisf_read.c
*
* This file contains the library function to read the In System Flash(ISF)
* device. Refer xilisf.h for detailed description.
*
* <pre>
*
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -----------------------------------------------
* 1.00a mta  07/11/07 First release.
*
* </pre>
*
******************************************************************************/

/***************************** Include Files *********************************/

#include "includes/xilisf.h"

/************************** Constant Definitions *****************************/

/**************************** Type Definitions *******************************/

/***************** Macros (Inline Functions) Definitions *********************/

/************************** Function Prototypes ******************************/

/************************** Variable Definitions *****************************/

/************************** Function Definitions ******************************/

extern int IsfTransfer(u8 *WritePtr, u8* ReadPtr, u32 ByteCount);

/*****************************************************************************/
/**
*
* This function reads the data from the ISF Memory
*
* @param	Addr is the starting address in the ISF Memory from which the
*		data is to be read.
* @param	ReadPtr is a pointer to the memory where data read from ISF is
*		stored.
* @param	ByteCount is the number of bytes need to read from the ISF.
*
* @return	XST_SUCCESS if successful else XST_FAILURE.
*
* @note		This works for frequencies less than 33Mhz. Minimum of one byte
*		and a maximum of an entire ISF array can be read.
* 		Read the Spartan-3AN In-System Flash User Guide for
*		more information. The valid data is available from the fourth
*		location pointed to by the ReadPtr.
*
******************************************************************************/
int XIsf_Read(XIsf_Address *Addr, u8 *ReadPtr, u32 ByteCount)
{
	u32 Index;
	u16 BytesTransfered = 0;
	int Status;

	if(IsfDev.IsReady != TRUE) {
		return XST_FAILURE;
	}

	if(Addr == NULL) {
		return XST_FAILURE;
	}

	if(ByteCount <= 0 ) {
		return XST_FAILURE;
	}

	if(Addr->SectorAddr > (IsfDev.NumOfSectors - 1)) {
		return XST_FAILURE;
	}

	if(Addr->BlockAddr > (IsfDev.BlocksPerSector - 1)) {
		return XST_FAILURE;
	}

	if(Addr->PageAddr > (IsfDev.PagesPerBlock - 1)) {
		return XST_FAILURE;
	}

	if(Addr->ByteAddr > (IsfDev.BytesPerPage - 1)) {
		return XST_FAILURE;
	}

	if(ReadPtr == NULL) {
		return XST_FAILURE;
	}

	/*
	 * Prepare the Write Buffer.
	 */
	ReadPtr[BYTE0] = XISF_COMMAND_RANDOM_READ;
	ReadPtr[BYTE1] = Addr->SectorAddr << XISF_SECTORADDR_BYTE_SHIFT_MASK
						| (Addr->BlockAddr >>
					XISF_BLOCKADDR_BYTE1_SHIFT_MASK);
	ReadPtr[BYTE2] = Addr->BlockAddr << XISF_BLOCKADDR_BYTE2_SHIFT_MASK
			 | Addr->PageAddr << XISF_PAGEADDR_BYTE2_SHIFT_MASK |
			 Addr->ByteAddr >> XISF_BYTEADDR_BYTE2_SHIFT_MASK;
	ReadPtr[BYTE3] = Addr->ByteAddr;

	/*
	 * Initiate the Transfer.
	 */
	Status = IsfTransfer(ReadPtr, ReadPtr,
				ByteCount + XISF_RANDOM_READ_EXTRA_BYTES);
	if(Status != XST_SUCCESS) {
		return XST_FAILURE;
	}

	return XST_SUCCESS;
}

