/* $Id: xilisf_spr.c 1.1 2006/02/17 21:21:36 mta Exp $ */
/******************************************************************************
*
*       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
*       SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
*       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
*       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
*       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*       FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*       FOR A PARTICULAR PURPOSE.
*
*       (c) Copyright 2007 Xilinx Inc.
*       All rights reserved.
*
******************************************************************************/
/*****************************************************************************/
/**
*
* @file xilisf_spr.c
*
* This file contains the library functions to operate on Sector Protection
* feature of the In System Flash(ISF) device. Refer xilisf.h for detailed
* description.
*
* <pre>
*
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -----------------------------------------------
* 1.00a mta  07/11/07 First release.
*
* </pre>
*
******************************************************************************/

/***************************** Include Files *********************************/

#include "includes/xilisf.h"

/************************** Constant Definitions *****************************/

/**************************** Type Definitions *******************************/

/***************** Macros (Inline Functions) Definitions *********************/

/************************** Function Prototypes ******************************/

extern int IsfTransfer(u8 *WritePtr, u8* ReadPtr,u32 ByteCount);

/************************** Variable Definitions *****************************/

/************************** Function Definitions ******************************/

/*****************************************************************************/
/**
*
* This function reads the content of the Sector Protection Register(SPR).
*
* @param	ReadPtr is a pointer to the memory where the SPR content is
*		copied.
*
* @return	XST_SUCCESS if successful else XST_FAILURE.
*
* @note		The SPR content is stored at the Fourth location pointed
*		by the ReadPtr.
*
******************************************************************************/
int XIsf_SprRead(u8 *ReadPtr)
{
	u32 Index;
	int Status;

	if(IsfDev.IsReady != TRUE) {
		return XST_FAILURE;
	}

	if(ReadPtr == NULL) {
		return XST_FAILURE;
	}

	/*
	 * Prepare the Write Buffer.
	 */
	IsfDev.IsfWriteBuffer[BYTE0] = XISF_COMMAND_SPR_READ;

	for(Index = 1; Index < (IsfDev.NumOfSectors + XISF_SPR_RDWR_BYTES);
								Index++) {
		IsfDev.IsfWriteBuffer[Index] = XISF_DUMMYBYTE; /* Dummy Bytes */
	}

	/*
	 * Initiate the Transfer.
	 */
	Status = IsfTransfer(IsfDev.IsfWriteBuffer, ReadPtr,
				IsfDev.NumOfSectors + XISF_SPR_RDWR_BYTES);
	if(Status != XST_SUCCESS) {
		return XST_FAILURE;
	}

	return XST_SUCCESS;
}

/*****************************************************************************/
/**
*
* This function erases the content of the Sector Protection Register(SPR).
*
* @param	None.
*
* @return	XST_SUCCESS if successful else XST_FAILURE.
*
* @note		None.
*
******************************************************************************/
int XIsf_SprErase(void)
{
	int Status;

	if(IsfDev.IsReady != TRUE) {
		return XST_FAILURE;
	}

	/*
	 * Prepare the Write Buffer.
	 */
	IsfDev.IsfWriteBuffer[BYTE0] = XISF_COMMAND_SPR_BYTE1;
	IsfDev.IsfWriteBuffer[BYTE1] = XISF_COMMAND_SPR_BYTE2;
	IsfDev.IsfWriteBuffer[BYTE2] = XISF_COMMAND_SPR_BYTE3;
	IsfDev.IsfWriteBuffer[BYTE3] = XISF_COMMAND_SPR_BYTE4_ERASE;

	/*
	 * Initiate the Transfer.
	 */
	Status = IsfTransfer(IsfDev.IsfWriteBuffer, NULL, XISF_SPR_ERASE_BYTES);
	if(Status != XST_SUCCESS) {
		return XST_FAILURE;
	}

	return XST_SUCCESS;
}

/*****************************************************************************/
/**
*
* This function writes to the Sector Protection Register(SPR).
*
* @param	BufferPtr is the pointer to the data to be written to SPR.
*
* @return	XST_SUCCESS if successful else XST_FAILURE..
*
* @note		None.
*
******************************************************************************/
int XIsf_SprProgram(u8 *BufferPtr)
{
	u32 Index;
	int Status;

	if(IsfDev.IsReady != TRUE) {
		return XST_FAILURE;
	}

	if(BufferPtr == NULL) {
		return XST_FAILURE;
	}

	/*
	 * Prepare the Write Buffer.
	 */
	IsfDev.IsfWriteBuffer[BYTE0] = XISF_COMMAND_SPR_BYTE1;
	IsfDev.IsfWriteBuffer[BYTE1] = XISF_COMMAND_SPR_BYTE2;
	IsfDev.IsfWriteBuffer[BYTE2] = XISF_COMMAND_SPR_BYTE3;
	IsfDev.IsfWriteBuffer[BYTE3] = XISF_COMMAND_SPR_BYTE4_PROGRAM;

	for(Index = 4; Index < (IsfDev.NumOfSectors + XISF_SPR_RDWR_BYTES);
								Index++) {
		IsfDev.IsfWriteBuffer[Index] = *BufferPtr++;
	}

	/*
	 * Initiate the Transfer.
	 */
	Status = IsfTransfer(IsfDev.IsfWriteBuffer, NULL,
				IsfDev.NumOfSectors + XISF_SPR_RDWR_BYTES);
	if(Status != XST_SUCCESS) {
		return XST_FAILURE;
	}

	return XST_SUCCESS;
}

/*****************************************************************************/
/**
*
* This function enables the Sector Protection. The protection is decided by the
* content of SPR register.
*
* @param	None.
*
* @return	XST_SUCCESS if successful else XST_FAILURE.
*
* @note		None.
*
******************************************************************************/
int XIsf_SpEnable(void)
{
	int Status;

	if(IsfDev.IsReady != TRUE) {
		return XST_FAILURE;
	}

	/*
	 * Prepare the Write Buffer.
	 */
	IsfDev.IsfWriteBuffer[BYTE0] = XISF_COMMAND_SPR_BYTE1;
	IsfDev.IsfWriteBuffer[BYTE1] = XISF_COMMAND_SPR_BYTE2;
	IsfDev.IsfWriteBuffer[BYTE2] = XISF_COMMAND_SPR_BYTE3;
	IsfDev.IsfWriteBuffer[BYTE3] = XISF_COMMAND_SPR_BYTE4_ENABLE;

	/*
	 * Initiate the Transfer.
	 */
	Status = IsfTransfer(IsfDev.IsfWriteBuffer, NULL, XISF_SPR_RDWR_BYTES);
	if(Status != XST_SUCCESS) {
		return XST_FAILURE;
	}

	return XST_SUCCESS;
}

/*****************************************************************************/
/**
*
* This function Disables the Sector Protection. The protection is decided by the
* content of SPR register.
*
* @param	None.
*
* @return	XST_SUCCESS if successful else XST_FAILURE.
*
* @note		None.
*
******************************************************************************/
int XIsf_SpDisable(void)
{
	int Status;

	if(IsfDev.IsReady != TRUE) {
		return XST_FAILURE;
	}

	/*
	 * Prepare the Write Buffer.
	 */
	IsfDev.IsfWriteBuffer[BYTE0] = XISF_COMMAND_SPR_BYTE1;
	IsfDev.IsfWriteBuffer[BYTE1] = XISF_COMMAND_SPR_BYTE2;
	IsfDev.IsfWriteBuffer[BYTE2] = XISF_COMMAND_SPR_BYTE3;
	IsfDev.IsfWriteBuffer[BYTE3] = XISF_COMMAND_SPR_BYTE4_DISABLE;

	/*
	 * Initiate the Transfer.
	 */
	Status = IsfTransfer(IsfDev.IsfWriteBuffer, NULL, XISF_SPR_RDWR_BYTES);
	if(Status != XST_SUCCESS) {
		return XST_FAILURE;
	}

	return XST_SUCCESS;
}
